@@ -13,6 +13,7 @@
 #     limitations under the License.
 #
 from functools import partial
+from functools import wraps
 from typing import Callable
 from typing import Dict
 from typing import List
@@ -20,6 +21,7 @@ from typing import Tuple
 from typing import Type
 from typing import Set
 from typing import Union
+import typing as t
 import contextlib
 import logging
 import sys
@@ -64,7 +66,6 @@ class FrescoApp(RouteCollection):
     request_class = Request
 
     def __init__(self, *args, **kwargs):
-
         views = kwargs.pop("views", None)
         path = kwargs.pop("path", None)
         super(FrescoApp, self).__init__(*args, **kwargs)
@@ -150,9 +151,9 @@ class FrescoApp(RouteCollection):
 
     def get_response(
         self,
-        request,
-        path,
-        method,
+        request: Request,
+        path: str,
+        method: str,
         currentcontext=context.currentcontext,
         normpath=normpath,
     ):
@@ -263,18 +264,14 @@ class FrescoApp(RouteCollection):
 
         return Response.not_found()
 
-    def view(self, request=None) -> Response:
+    def view(self, request: t.Optional[Request] = None) -> Response:
         request = request or context.request
         try:
             path = request.path_info
         except ResponseException as e:
             response = e.response
         else:
-            response = self.get_response(
-                request,
-                path,
-                request.method
-            )
+            response = self.get_response(request, path, request.method)
 
         for f in self.process_response_handlers:
             try:
@@ -331,7 +328,6 @@ class FrescoApp(RouteCollection):
         Response,
         Tuple[Type[BaseException], BaseException, types.TracebackType],
     ]:
-
         exc_info = sys.exc_info()
         if exc_info[0] is None:
             raise AssertionError(
@@ -450,8 +446,7 @@ class FrescoApp(RouteCollection):
             iterator = None
             try:
                 iterator = wsgi_app(environ, start_response)
-                for item in iterator:
-                    yield item
+                yield from iterator
             except Exception:
                 exc_info = sys.exc_info()
                 try:
@@ -464,14 +459,15 @@ class FrescoApp(RouteCollection):
                     def exc_start_response(s, h, exc_info=exc_info):
                         return start_response(s, h, exc_info)
 
-                    for item in response(environ, exc_start_response):
-                        yield item
+                    yield from response(environ, exc_start_response)
                 finally:
                     del exc_info
             finally:
                 try:
                     if process_teardown_handlers:
                         call_process_teardown_handlers(request)
+                    for item in request.teardown_handlers:
+                        item()
                 finally:
                     try:
                         close = getattr(iterator, "close", None)
@@ -562,6 +558,27 @@ class FrescoApp(RouteCollection):
             except Exception:
                 self.log_exception(request)
 
+    def process_request_once(
+        self, func: Callable[[Request], t.Optional[Response]]
+    ) -> Callable[[Request], t.Optional[Response]]:
+        """
+        Register a ``process_request`` hook function that is called only once
+
+        When running fresco with multiple worker threads/processes the hook
+        function will be called at most once per worker.
+        """
+
+        @self.process_request
+        @wraps(func)
+        def process_request_once(request: Request) -> t.Optional[Response]:
+            try:
+                self.process_request_handlers.remove(process_request_once)
+            except ValueError:
+                return None
+            return func(request)
+
+        return func
+
     def process_request(self, func):
         """
         Register a ``process_request`` hook function
@@ -655,17 +672,19 @@ class FrescoApp(RouteCollection):
         environ = make_environ(url, environ, wsgi_input, **kwargs)
         app = self.make_wsgi_app(wsgi_app=fake_app, use_middleware=middleware)
         result = app(environ, fake_start_response)
-        close = getattr(result, "close", lambda: None)
+        close = getattr(result, "close", None)
         content_iterator = iter(result)
-        next(content_iterator, None)
-        yield context
-        list(content_iterator)
-        close()
+        try:
+            next(content_iterator, None)
+            yield context
+            list(content_iterator)
+        finally:
+            if close is not None:
+                close()
 
     def requestcontext_with_payload(
         self, url="/", data=None, environ=None, files=None, multipart=False, **kwargs
     ):
-
         if files:
             multipart = True
 
@@ -676,6 +695,8 @@ class FrescoApp(RouteCollection):
             wsgi_input = data.read()
         elif isinstance(data, bytes):
             wsgi_input = data
+        elif data is None:
+            wsgi_input = ""
         else:
             wsgi_input = make_query(data).encode("ascii")
 
