@@ -79,9 +79,7 @@ def json_response(
         def json_response_decorator(func):
             @wraps(func)
             def json_response_decorated(*fa, **fkw):
-                return Response.json(
-                    func(*fa, **fkw), indent, separators, **kwargs
-                )
+                return Response.json(func(*fa, **fkw), indent, separators, **kwargs)
 
             return json_response_decorated
 
