@@ -12,7 +12,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-__all__ = "XForwarded"
+__all__ = ["XForwarded"]
 
 
 class XForwarded(object):
@@ -100,7 +100,6 @@ class XForwarded(object):
             environ["HTTPS"] = "on"
 
         try:
-
             forwards = environ["HTTP_X_FORWARDED_FOR"].split(", ") + [
                 env("REMOTE_ADDR", "")
             ]
