@@ -16,12 +16,24 @@
 An order preserving multidict implementation
 """
 from collections.abc import MutableMapping
+from collections.abc import Mapping
 from itertools import repeat
+import typing as t
 from typing import Any
 from typing import Dict
 from typing import Set
 
 
+if t.TYPE_CHECKING:
+    from typeshed import SupportsKeysAndGetItem
+else:
+    SupportsKeysAndGetItem = Mapping
+
+SupportsKeysAndGetItemOrIterable = t.Union[
+    SupportsKeysAndGetItem, t.Iterable[t.Tuple[Any, Any]]
+]
+
+
 class MultiDict(MutableMapping):
     """
     Dictionary-like object that supports multiple values per key. Insertion
@@ -44,9 +56,13 @@ class MultiDict(MutableMapping):
 
     """
 
+    _dict: Dict[Any, Any]
+    _order: t.List[Any]
     setdefault = MutableMapping.setdefault
 
-    def __init__(self, *args, **kwargs):
+    def __init__(
+        self, mapping_or_iterable: SupportsKeysAndGetItemOrIterable = tuple(), **kwargs
+    ):
         """
         MultiDicts can be constructed in the following ways:
 
@@ -76,13 +92,9 @@ class MultiDict(MutableMapping):
                 MultiDict(...)
 
         """
-        if len(args) > 1:
-            raise TypeError(
-                "%s expected at most 2 arguments, got %d"
-                % (self.__class__.__name__, 1 + len(args))
-            )
-        self.clear()
-        self.update(*args, **kwargs)
+        self._order = []
+        self._dict = {}
+        self._update(mapping_or_iterable, True, kwargs)
 
     def __getitem__(self, key):
         """
@@ -289,7 +301,7 @@ class MultiDict(MutableMapping):
         self._order.remove((key, value))
         return value
 
-    def popitem(self):
+    def popitem(self) -> t.Tuple[Any, Any]:
         """
         Return and remove a ``(key, value)`` pair from the dictionary.
 
@@ -314,7 +326,17 @@ class MultiDict(MutableMapping):
             raise KeyError("popitem(): dictionary is empty")
         return key, self.pop(key)
 
-    def update(self, *args, **kwargs):
+    @t.overload
+    def update(
+        self, mapping_or_iterable: SupportsKeysAndGetItem, /, **kwargs: Any
+    ) -> None:
+        ...
+
+    @t.overload
+    def update(self, /, **kwargs: Any) -> None:
+        ...
+
+    def update(self, mapping_or_iterable=tuple(), **kwargs):
         r"""
         Update the MultiDict from another MultiDict, regular dictionary or a
         iterable of ``(key, value)`` pairs. New keys overwrite old keys - use
@@ -349,33 +371,24 @@ class MultiDict(MutableMapping):
                 >>> d.update(mood='okay')
 
         """
-        if len(args) > 1:
-            raise TypeError(
-                "expected at most 1 argument, got %d" % (1 + len(args),)
-            )
-        if args:
-            other = args[0]
-        else:
-            other = []
-        return self._update(other, True, **kwargs)
+        self._update(mapping_or_iterable, True, kwargs)
 
-    def extend(self, *args, **kwargs):
+    def extend(
+        self, mapping_or_iterable: SupportsKeysAndGetItemOrIterable = tuple(), **kwargs
+    ):
         """
         Extend the MultiDict with another MultiDict, regular dictionary or a
         iterable of ``(key, value)`` pairs. This is similar to :meth:`update`
         except that new keys are added to old keys.
         """
-        if len(args) > 1:
-            raise TypeError(
-                "expected at most 1 argument, got %d", (1 + len(args),)
-            )
-        if args:
-            other = args[0]
-        else:
-            other = []
-        return self._update(other, False, **kwargs)
+        return self._update(mapping_or_iterable, False, kwargs)
 
-    def _update(self, *args, **kwargs):
+    def _update(
+        self,
+        other: SupportsKeysAndGetItemOrIterable,
+        replace: bool,
+        extra_kwargs: Dict[Any, Any],
+    ):
         """
         Update the MultiDict from another object and optionally kwargs.
 
@@ -384,29 +397,27 @@ class MultiDict(MutableMapping):
         :param replace: if ``True``, entries will replace rather than extend
                         existing entries (second positional arg)
         """
-        other, replace = args
-
         if isinstance(other, self.__class__):
             items = list(other.allitems())
-        elif isinstance(other, dict):
+        elif isinstance(other, Mapping):
             items = list(other.items())
         else:
             items = list(other)
 
-        if kwargs:
-            items += list(kwargs.items())
+        if extra_kwargs:
+            items += list(extra_kwargs.items())
 
         if replace:
             replaced = {k for k, v in items if k in self._dict}
-            self._order = [
-                (k, v) for (k, v) in self._order if k not in replaced
-            ]
+            self._order = [(k, v) for (k, v) in self._order if k not in replaced]
             for key in replaced:
                 self._dict[key] = []
 
+        setdefault = self._dict.setdefault
+        append_order = self._order.append
         for k, v in items:
-            self._dict.setdefault(k, []).append(v)
-            self._order.append((k, v))
+            setdefault(k, []).append(v)
+            append_order((k, v))
 
     def __repr__(self):
         """
@@ -434,4 +445,4 @@ class MultiDict(MutableMapping):
 
     def clear(self):
         self._order = []
-        self._dict: Dict[Any, Any] = {}
+        self._dict = {}
