@@ -15,6 +15,7 @@
 import inspect
 import json
 import logging
+import typing as t
 import os
 import re
 from decimal import Decimal
@@ -31,6 +32,8 @@ from typing import Union
 
 from fresco.exceptions import OptionsLoadedException
 
+__all__ = ["Options"]
+
 logger = logging.getLogger(__name__)
 
 
@@ -72,25 +75,39 @@ class Options(dict):
         for func in self._loaded_callbacks:
             func(self)
 
+    def trigger_onload(self):
+        """
+        Mark the options object as having loaded and call any registered
+        onload callbacks
+        """
+        if self._is_loaded:
+            raise OptionsLoadedException("Options have already been loaded")
+        self.do_loaded_callbacks()
+        self.__dict__["_is_loaded"] = True
+
     def copy(self):
         return self.__class__(super().copy())
 
     def load(
         self,
-        sources: str,
+        sources: t.Union[str, t.Iterable[t.Union[Path, str]]],
         tags: Sequence[str] = [],
         use_environ=False,
         strict=True,
         dir=None,
+        trigger_onload=True,
     ):
         """
         Find all files matching glob pattern ``sources`` and populates the
         options object from those with matching filenames containing ``tags``.
 
-        :param sources: glob pattern or glob patterns separated by ";"
-        :param tags: a list of tags to look for in file names. If a filename
-                     contains multiple tags, all the tags in the filename must
-                     match for it to be loaded.
+        :param sources:
+            one or more glob patterns separated by ";",
+            or a list of glob patterns
+
+        :param tags:
+            a list of tags to look for in file names.
+
         :param use_environ: if true, environment variables matching previously
                             loaded keys will be loaded into the options object.
                             This happens after all files have been processed.
@@ -102,13 +119,21 @@ class Options(dict):
         format. Any other files will be interpreted as simple lists of
         ```key=value`` pairs.
 
-        Tags in filenames should be delimited with periods, eg ".env.production.py". For
-        For example the filename ``setttings.dev.local.ini`` would be
-        considered to have the tags, ``('dev', 'local')``
+        Tags in filenames are delimited with periods,
+        for example ".env.production.py".
+        The filename ``setttings.dev.local.ini`` would be
+        considered to have the tags ``('dev', 'local')``
 
-        The string '{hostname}' can be included as part of a tag name: it will
-        be substituted for the current host's name with dots replaced by
-        underscores.
+        Where filename contain multiple tags, all tags must match for the file
+        to be loaded.
+
+        Tag names may contain the names of environment variable surrounded by
+        braces, for example ``{USER}``. These will be substituted for the
+        environment variable's value, with any dots or path separators replaced
+        by underscores.
+
+        The special variable ``{hostname}`` will be substituted for the current
+        host's name with dots replaced by underscores.
 
         Files with the suffix ".sample" are unconditionally excluded.
 
@@ -118,7 +143,7 @@ class Options(dict):
         Example::
 
             opts = Options()
-            opts.load(Options(), ".env*", ["dev", "host-{hostname}", "local"])
+            opts.load(".env*", ["dev", "host-{hostname}", "local"])
 
         Would load options from files named ``.env``, ``.env.json``, ``.env.dev.py``
         and ``.env.local.py.``
@@ -126,15 +151,24 @@ class Options(dict):
         """
         if self._is_loaded:
             raise OptionsLoadedException("Options have already been loaded")
-        hostname = gethostname().replace(".", "_")
+
+        tag_substitutions = os.environ.copy()
+        tag_substitutions["hostname"] = gethostname()
+        tag_substitutions = {
+            k: v.replace(".", "_").replace(os.pathsep, "_")
+            for k, v in tag_substitutions.items()
+        }
+
         candidates: List[Path] = []
         if dir is None:
             dir = Path(".")
         else:
             dir = Path(dir)
 
-        for source in sources.split(";"):
-            sourcepath = dir / Path(source.strip())
+        if isinstance(sources, str):
+            sources = [s.strip() for s in sources.split(";")]
+        for source in sources:
+            sourcepath = dir / Path(source)
             candidates.extend(
                 p
                 for p in sourcepath.parent.glob(sourcepath.name)
@@ -142,7 +176,13 @@ class Options(dict):
             )
 
         if tags:
-            tags = [t.format(hostname=hostname) for t in tags]
+            subbed_tags = []
+            for tag in tags:
+                try:
+                    subbed_tags.append(tag.format(**tag_substitutions))
+                except KeyError:
+                    pass
+            tags = subbed_tags
             tagged_sources = []
             for p in candidates:
                 candidate_tags = [t for t in str(p.name).split(".") if t][1:]
@@ -216,8 +256,9 @@ class Options(dict):
                     if k in os.environ:
                         self[k] = parse_value(self, os.environ[k])
 
-        self.do_loaded_callbacks()
-        self.__dict__["_is_loaded"] = True
+        if trigger_onload:
+            self.do_loaded_callbacks()
+            self.__dict__["_is_loaded"] = True
         return self
 
     def update_from_file(self, path, load_all=False):
@@ -256,9 +297,11 @@ class Options(dict):
 
     def update_from_object(self, ob, load_all=False):
         """
-        Update the instance with any symbols listed in object `ob`
-        :param load_all: If true private symbols will also be loaded into the
-                         options object.
+        Update the instance with any symbols found in object ``ob``.
+
+        :param load_all:
+            If true private symbols will also be loaded into the options
+            object.
         """
         self.update_from_dict(dict(inspect.getmembers(ob)), load_all)
 
@@ -298,3 +341,70 @@ def parse_key_value_pairs(options, lines: Iterable[str]):
         k = k.strip()
         values[k] = options[k] = parse_value(options, v)
     return values
+
+
+def list_from_str(s, separator=","):
+    """
+    Return the value of ``s`` split into a list of times by ``separator``.
+    Spaces around items will be stripped.
+    """
+    if isinstance(s, str):
+        if not s.strip():
+            return []
+        return [item.strip() for item in s.split(",")]
+    raise TypeError(f"Expected string, got {s!r}")
+
+
+def dict_from_options(
+    prefix: str, options: t.Mapping[str, Any], recursive: bool = False
+) -> t.Dict[str, Any]:
+    """
+    Create a dictionary containing all items in ``options`` whose keys start
+    with ``prefix``, with that prefix stripped.
+
+    Example:
+
+        >>> from fresco.options import Options, dict_from_options
+        >>> options = Options(
+        ...      {"DATABASE_HOST": "127.0.0.1", "DATABASE_USER": "scott"}
+        ... )
+        >>> dict_from_options("DATABASE_", options)
+        {'HOST': '127.0.0.1', 'USER': 'scott'}
+
+    If ``recursive`` is True, a recursive splitting will be
+    applied. The last character of ``prefix`` will be used as the separator,
+    for example::
+
+        >>> from fresco.options import dict_from_options
+        >>> options = Options(
+        ...      {
+        ...          "DATABASE_DEV_HOST": "127.0.0.1",
+        ...          "DATABASE_DEV_USER": "scott",
+        ...          "DATABASE_PROD_HOST": "192.168.0.78",
+        ...          "DATABASE_PROD_HOST": "tiger",
+        ...      }
+        ... )
+        >>> dict_from_options("DATABASE_", options, recursive=True)
+        {'DEV': {'HOST': '127.0.0.1', ...}, 'PROD': {'HOST', ...}}
+
+    """
+
+    prefixlen = len(prefix)
+    d = {k[prefixlen:]: v for k, v in options.items() if k.startswith(prefix)}
+
+    def recursive_split(d: t.Mapping[str, Any], sep: str) -> t.Dict[str, Any]:
+        d_: t.Dict[str, Any] = {}
+        for k in d:
+            if sep in k:
+                ks = k.split(sep)
+                subdict = d_
+                for subkey in ks[:-1]:
+                    subdict = subdict.setdefault(subkey, {})
+                subdict[ks[-1]] = d[k]
+            else:
+                d_[k] = d[k]
+        return d_
+
+    if recursive:
+        return recursive_split(d, prefix[-1])
+    return d
