@@ -23,8 +23,7 @@ from urllib.parse import ParseResult
 from urllib.parse import quote
 from urllib.parse import urlparse
 from urllib.parse import urlunparse
-from typing import Dict
-from typing import Optional
+import typing as t
 import datetime
 import json
 import posixpath
@@ -33,6 +32,7 @@ import re
 from fresco import exceptions
 from fresco.cookie import parse_cookie_header
 from fresco.multidict import MultiDict
+from fresco.types import QuerySpec
 from fresco.util.http import FileUpload
 from fresco.util.http import get_body_bytes
 from fresco.util.http import get_content_type_info
@@ -83,7 +83,7 @@ class Request(object):
     SESSION_ENV_KEY = "beaker.session"
 
     #: The WSGI environ dict
-    environ: Dict
+    environ: t.Dict
 
     #: Encoding used to decode WSGI parameters, notably PATH_INFO and form data
     default_charset = fresco.DEFAULT_CHARSET
@@ -91,12 +91,16 @@ class Request(object):
     #: The decoder class to use for JSON request payloads
     json_decoder_class = json.JSONDecoder
 
+    #: List of functions to be called at the end of this request's lifecycle
+    teardown_handlers: t.List[t.Callable]
+
     def __init__(self, environ):
         if self.STATE_ENV_KEY in environ:
             self.__dict__ = environ[self.STATE_ENV_KEY]
         else:
             environ[self.STATE_ENV_KEY] = self.__dict__
-        self.environ = environ
+            self.environ = environ
+            self.teardown_handlers = []
 
     def __str__(self):
         """
@@ -133,17 +137,18 @@ class Request(object):
         the ``query`` property.
         """
         if self._form is None:
-            if self.environ["REQUEST_METHOD"] in ("PUT", "POST"):
-                self._form = MultiDict(
-                    parse_post(
-                        self.environ,
-                        self.environ["wsgi.input"],
-                        self.charset,
-                        self.MAX_SIZE,
-                        self.MAX_MULTIPART_SIZE,
-                        ie_workaround=self.IE_CONTENT_DISPOSITION_WORKAROUND,
-                    )
+            if self.environ["REQUEST_METHOD"] in {"PUT", "POST"}:
+                items, close = parse_post(
+                    self.environ,
+                    self.environ["wsgi.input"],
+                    self.charset,
+                    self.MAX_SIZE,
+                    self.MAX_MULTIPART_SIZE,
+                    ie_workaround=self.IE_CONTENT_DISPOSITION_WORKAROUND,
                 )
+                if close:
+                    self.teardown_handlers.append(close)
+                self._form = MultiDict(items)
             else:
                 data = environ_to_str(self.environ.get("QUERY_STRING", ""))
                 self._form = MultiDict(parse_querystring(data, self.charset))
@@ -470,7 +475,7 @@ class Request(object):
             raise exceptions.BadRequest
 
     @property
-    def query_string(self) -> Optional[str]:
+    def query_string(self) -> t.Optional[str]:
         """
         The QUERY_STRING value as a unicode string
         """
@@ -522,19 +527,19 @@ class Request(object):
 
     def make_url(
         self,
-        scheme=None,
-        netloc=None,
-        path=None,
-        parameters=None,
-        query=None,
-        query_add=None,
-        query_replace=None,
-        fragment=None,
-        SCRIPT_NAME=None,
-        PATH_INFO=None,
+        scheme: t.Optional[str] = None,
+        netloc: t.Optional[str] = None,
+        path: t.Optional[str] = None,
+        parameters: t.Optional[str] = None,
+        query: t.Union[None, str, QuerySpec] = None,
+        query_add: t.Union[None, str, QuerySpec] = None,
+        query_replace: t.Union[None, str, QuerySpec] = None,
+        fragment: t.Optional[str] = None,
+        SCRIPT_NAME: t.Optional[str] = None,
+        PATH_INFO: t.Optional[str] = None,
         **kwargs,
-    ):
-        r"""\
+    ) -> str:
+        r"""
         Make a new URL based on the current URL, replacing any of the six
         URL elements (scheme, netloc, path, parameters, query or fragment). The
         current request's query string is not included in the generated URL
@@ -547,8 +552,8 @@ class Request(object):
                             urlparse module)
         :param fragment:    The URL fragment
         :param query:       Query data, as a list of tuples, a dict or a string.
-                            Any query parameters present in the current url
-                            will be removed.
+                            If supplied, any query parameters present in the
+                            current request's url will be removed.
         :param query_add:   Query parameters to add. If the current url already
                             contains query parameters with the same name, these
                             will be supplemented with the new values.
@@ -635,6 +640,7 @@ class Request(object):
         other type convertable to a MultiDict.
         """
         parsed_url = self.parsed_url
+        querystr: t.Optional[str] = None
 
         if path is not None:
             path = quote(path, encoding=self.charset)
@@ -676,17 +682,20 @@ class Request(object):
             if kwargs:
                 query_dict.update(kwargs)
 
-            query = make_query(query_dict)
+            querystr = make_query(query_dict)
 
-        elif query is not None and not isinstance(query, str):
-            query = make_query(query)
+        elif query is not None:
+            if isinstance(query, str):
+                querystr = query
+            else:
+                querystr = make_query(query)
 
         url = (
             scheme if scheme is not None else parsed_url[0],
             netloc if netloc is not None else parsed_url[1],
             path if path is not None else parsed_url[2],
             parameters if parameters is not None else parsed_url[3],
-            query,
+            querystr,
             fragment,
         )
 
