@@ -16,13 +16,16 @@ from collections import defaultdict
 from threading import get_ident
 from typing import Any
 from typing import Dict
-from typing import List
+import typing as t
 
-__all__ = "context", "RequestContext"
+__all__ = ["context", "RequestContext"]
+
+ContextDict = t.Dict[str, Any]
+ContextStack = t.List[ContextDict]
 
 
 class RequestContext(object):
-    """\
+    """
     A local storage class that maintains different values for each request
     context in which it is used.
 
@@ -42,13 +45,17 @@ class RequestContext(object):
 
     __slots__ = ["_contexts", "_ident_func"]
 
-    def __init__(self, ident_func=get_ident, setattr=object.__setattr__):
-        c: Dict[Any, List] = defaultdict(list)
+    def __init__(
+        self,
+        ident_func: t.Callable[[], t.Any] = get_ident,
+        setattr=object.__setattr__
+    ):
+        c: Dict[Any, ContextStack] = defaultdict(list)
         c[ident_func()] = [{}]
         setattr(self, "_contexts", c)
         setattr(self, "_ident_func", ident_func)
 
-    def push(self, **bindnames):
+    def push(self, **bindnames: t.Any):
         self._contexts[self._ident_func()].append(bindnames)
 
     def pop(self):
@@ -58,7 +65,7 @@ class RequestContext(object):
         if not ctx:
             del self._contexts[ident]
 
-    def currentcontext(self):
+    def currentcontext(self) -> ContextDict:
         return self._contexts[self._ident_func()][-1]
 
     def __getattr__(self, item):
@@ -67,7 +74,7 @@ class RequestContext(object):
         except (KeyError, IndexError):
             raise AttributeError(item)
 
-    def __setattr__(self, item, ob):
+    def __setattr__(self, item: str, ob: t.Any):
         self._contexts[self._ident_func()][-1][item] = ob
 
     def __delattr__(self, item):
@@ -81,7 +88,7 @@ class RequestContext(object):
     __setitem__ = __setattr__
     __delitem__ = __delattr__
 
-    def get(self, item, default=None):
+    def get(self, item: str, default=None):
         return self.currentcontext().get(item, default)
 
     def __repr__(self):
