@@ -261,7 +261,7 @@ default_charset = "UTF-8"
 
 
 def encoder(stream, charset):
-    r"""\
+    """
     Encode a response iterator using the given character set.
     """
     if charset is None:
@@ -326,7 +326,7 @@ def make_headers(
 
 
 class Response(object):
-    """\
+    """
     Model an HTTP response
     """
 
@@ -342,7 +342,7 @@ class Response(object):
         passthrough=False,
         content_iterator=None,
         make_headers=make_headers,
-        **kwargs
+        **kwargs,
     ):
         """
         Create a new Response object, modelling the HTTP status, headers and
@@ -406,9 +406,7 @@ class Response(object):
         # provided
         if content is not _nocontent:
             if not any(k == "Content-Type" for k, v in headers):
-                headers = [
-                    ("Content-Type", self.default_content_type)
-                ] + headers
+                headers = [("Content-Type", self.default_content_type)] + headers
 
         self.headers = headers
 
@@ -422,7 +420,6 @@ class Response(object):
         # code assumes that lists are homogeneous.
         self.content = content
         if content_iterator:
-            self.content = content
             self.content_iterator = content_iterator
         elif passthrough or content is _nocontent:
             self.content_iterator = content
@@ -442,9 +439,7 @@ class Response(object):
             elif isinstance(content, str):
                 self.content_iterator = [content.encode(self.charset)]
             else:
-                self.content_iterator = _copy_close(
-                    content, encoder(content, self.charset)
-                )
+                self.content_iterator = encoder(content, self.charset)
 
     def __call__(self, environ, start_response, exc_info=None):
         """
@@ -463,7 +458,11 @@ class Response(object):
         method.
         """
         return self.__class__(
-            self.content, self.status, self.headers, self.onclose + list(funcs)
+            self.content,
+            self.status,
+            self.headers,
+            self.onclose + list(funcs),
+            content_iterator=self.content_iterator,
         )
 
     @classmethod
@@ -475,10 +474,9 @@ class Response(object):
         """
         responder = StartResponseWrapper(start_response)
         content = wsgi_callable(environ, responder)
+        close = getattr(content, "close", None)
         if responder.buf.tell():
-            content = _copy_close(
-                content, chain(content, [responder.buf.getvalue()])
-            )
+            content = chain(content, [responder.buf.getvalue()])
 
         if not responder.called:
             # Iterator has not called start_response yet. Call next(content)
@@ -488,21 +486,21 @@ class Response(object):
             except StopIteration:
                 pass
             except Exception:
-                close = getattr(content, "close", None)
                 if close is not None:
                     close()
                 raise
             else:
-                content = _copy_close(content, chain([chunk], content))
+                content = chain([chunk], content)
         return cls(
             content,
             responder.status,
             headers=responder.headers,
             passthrough=True,
+            onclose=[close] if close is not None else [],
         )
 
     def get_headers(self, name):
-        """\
+        """
         Return the list of headers set with the given name.
 
         Synopsis::
@@ -513,13 +511,11 @@ class Response(object):
 
         """
         return [
-            value
-            for header, value in self.headers
-            if header.lower() == name.lower()
+            value for header, value in self.headers if header.lower() == name.lower()
         ]
 
     def get_header(self, name, default=""):
-        """\
+        """
         Return the concatenated values of the named header(s) or ``default`` if
         the header has not been set.
 
@@ -551,7 +547,7 @@ class Response(object):
 
     @property
     def content_type(self):
-        """\
+        """
         Return the value of the ``Content-Type`` header if set, otherwise
         ``None``.
         """
@@ -561,7 +557,7 @@ class Response(object):
         return None
 
     def add_header(self, name, value, make_header_name=make_header_name):
-        """\
+        """
         Return a new response object with the given additional header.
 
         Synopsis::
@@ -572,12 +568,10 @@ class Response(object):
             >>> r.add_header('Cache-Control', 'no-cache').headers
             [('Content-Type', 'text/plain'), ('Cache-Control', 'no-cache')]
         """
-        return self.replace(
-            headers=(self.headers + [(make_header_name(name), value)])
-        )
+        return self.replace(headers=(self.headers + [(make_header_name(name), value)]))
 
     def add_headers(self, headers=[], **kwheaders):
-        """\
+        """
         Return a new response object with the given additional headers.
 
         Synopsis::
@@ -590,12 +584,10 @@ class Response(object):
             ... ).headers
             [('Content-Type', 'text/plain'), ('Cache-Control', 'no-cache')]
         """
-        return self.replace(
-            headers=make_headers(self.headers + headers, kwheaders)
-        )
+        return self.replace(headers=make_headers(self.headers + headers, kwheaders))
 
     def remove_headers(self, *headers):
-        """\
+        """
         Return a new response object with the named headers removed.
 
         Synopsis::
@@ -625,7 +617,7 @@ class Response(object):
         httponly=False,
         samesite="Lax",
     ):
-        """\
+        """
         Return a new response object with the given cookie added.
 
         Synopsis::
@@ -680,9 +672,7 @@ class Response(object):
                 _vary_on.update(s.strip() for s in v.split(","))
             else:
                 newheaders.append((k, v))
-        return self.replace(
-            headers=newheaders + [("Vary", ", ".join(_vary_on))]
-        )
+        return self.replace(headers=newheaders + [("Vary", ", ".join(_vary_on))])
 
     def replace(self, content=None, status=None, headers=None, **kwheaders):
         """\
@@ -717,9 +707,9 @@ class Response(object):
 
         if kwheaders:
             toremove = {make_header_name(k) for k in kwheaders}
-            headers = [
-                (k, v) for k, v in headers if k not in toremove
-            ] + make_headers([], kwheaders)
+            headers = [(k, v) for k, v in headers if k not in toremove] + make_headers(
+                [], kwheaders
+            )
 
         return self.__class__(
             content,
@@ -730,7 +720,7 @@ class Response(object):
         )
 
     def buffered(self):
-        """\
+        """
         Return a new response object with the content buffered into a list.
         This will also generate a content-length header.
 
@@ -766,7 +756,7 @@ class Response(object):
 
     @classmethod
     def not_found(cls, request=None):
-        """\
+        """
         Return an HTTP not found response (404).
 
         Synopsis::
@@ -821,7 +811,7 @@ class Response(object):
             "</body>\n"
             "</html>",
             status=STATUS_UNAUTHORIZED,
-            www_authenticate=authenticate
+            www_authenticate=authenticate,
         )
 
     @classmethod
@@ -834,7 +824,7 @@ class Response(object):
 
     @classmethod
     def forbidden(cls, message="Sorry, access is denied"):
-        """\
+        """
         Return an HTTP forbidden response (403).
 
         Synopsis::
@@ -844,17 +834,13 @@ class Response(object):
             ...
         """
         return cls(
-            "<html>\n"
-            "<body>\n"
-            "<h1>%s</h1>\n"
-            "</body>\n"
-            "</html>" % (message,),
+            "<html>\n<body>\n<h1>%s</h1>\n</body>\n</html>" % (message,),
             status=STATUS_FORBIDDEN,
         )
 
     @classmethod
     def bad_request(cls, request=None):
-        """\
+        """
         Return an HTTP bad request response.
 
         Synopsis::
@@ -877,7 +863,7 @@ class Response(object):
 
     @classmethod
     def length_required(cls, request=None):
-        """\
+        """
         Return an HTTP Length Required response (411).
 
         Synopsis::
@@ -900,7 +886,7 @@ class Response(object):
 
     @classmethod
     def payload_too_large(cls, request=None):
-        """\
+        """
         Return an HTTP Payload Too Large response (413)::
 
             >>> response = Response.payload_too_large()
@@ -908,20 +894,14 @@ class Response(object):
         """
         return cls(
             status=STATUS_PAYLOAD_TOO_LARGE,
-            content=[
-                "<html>"
-                "<body>"
-                "<h1>Payload Too Large</h1>"
-                "</body>"
-                "</html>"
-            ],
+            content=["<html><body><h1>Payload Too Large</h1></body></html>"],
         )
 
     request_entity_too_large = payload_too_large
 
     @classmethod
     def method_not_allowed(cls, valid_methods):
-        """\
+        """
         Return an HTTP method not allowed response (405)::
 
             >>> from fresco import context
@@ -938,18 +918,12 @@ class Response(object):
         return cls(
             status=STATUS_METHOD_NOT_ALLOWED,
             allow=",".join(valid_methods),
-            content=[
-                "<html>"
-                "<body>"
-                "<h1>Method not allowed</h1>"
-                "</body>"
-                "</html>"
-            ],
+            content=["<html><body><h1>Method not allowed</h1></body></html>"],
         )
 
     @classmethod
     def internal_server_error(cls):
-        """\
+        """
         Return an HTTP internal server error response (500).
 
         Synopsis::
@@ -963,20 +937,14 @@ class Response(object):
 
         return cls(
             status=STATUS_INTERNAL_SERVER_ERROR,
-            content=[
-                "<html>"
-                "<body>"
-                "<h1>Internal Server Error</h1>"
-                "</body>"
-                "</html>"
-            ],
+            content=["<html><body><h1>Internal Server Error</h1></body></html>"],
         )
 
     @classmethod
     def unrestricted_redirect(
         cls, location, request=None, status=STATUS_FOUND, **kwargs
     ):
-        """\
+        """
         Return an HTTP redirect reponse (30x).
 
         :param location: The redirect location or a view specification.
@@ -1076,7 +1044,7 @@ class Response(object):
         status=STATUS_FOUND,
         _is_safe_url=is_safe_url,
         allowed_hosts=frozenset(),
-        **kwargs
+        **kwargs,
     ):
         """
         Return an HTTP redirect reponse (30x). Will only redirect to the
@@ -1092,7 +1060,8 @@ class Response(object):
         :param kwargs: kwargs to be passed to :func:`fresco.core.urlfor` to
                        construct the redirect URL, or the fallback in the case
                        that ``location`` is already a qualified URL.
-        Synopsis:
+
+        Synopsis::
 
             >>> def view():
             ...   return Response.redirect("/new-location")
@@ -1104,9 +1073,7 @@ class Response(object):
         # Create the fallback redirect response always so that
         # changes to route names don't result in latent bugs
         if fallback:
-            fallback = Response.unrestricted_redirect(
-                fallback, status=status, **kwargs
-            )
+            fallback = Response.unrestricted_redirect(fallback, status=status, **kwargs)
         if callable(url) or "//" not in url:
             return cls.unrestricted_redirect(url, status=status, **kwargs)
 
@@ -1118,7 +1085,7 @@ class Response(object):
 
     @classmethod
     def redirect_permanent(cls, *args, **kwargs):
-        """\
+        """
         Return an HTTP permanent redirect reponse.
 
         :param location: the URI of the new location. If relative this will be
@@ -1131,7 +1098,7 @@ class Response(object):
 
     @classmethod
     def redirect_temporary(cls, *args, **kwargs):
-        """\
+        """
         Return an HTTP permanent redirect reponse.
 
         :param location: the URI of the new location. If relative this will be
@@ -1144,7 +1111,7 @@ class Response(object):
 
     @classmethod
     def meta_refresh(cls, location, delay=1, request=None):
-        """\
+        """
         Return an HTML page containing a <meta http-equiv="refresh"> tag,
         causing the browser to redirect to the given location after ``delay``
         seconds.
@@ -1181,7 +1148,7 @@ class Response(object):
         status=None,
         headers=None,
         dumps=stdlib_json.dumps,
-        **kwargs
+        **kwargs,
     ):
         """
         Return an ``application/json`` response with the given data
@@ -1206,7 +1173,7 @@ class Response(object):
 
 
 def dump_response(r, line_break=b"\r\n", encoding="UTF-8"):
-    """\
+    """
     Return a byte-string representation of the given response, as for a HTTP
     response.
     """
@@ -1224,24 +1191,3 @@ def dump_response(r, line_break=b"\r\n", encoding="UTF-8"):
     if encoding:
         return s.decode(encoding)
     return s
-
-
-def _copy_close(src, dst, marker=object()):
-    """\
-    Copy the ``close`` attribute from ``src`` to ``dst``, which are assumed to
-    be iterators.
-
-    If it is not possible to copy the attribute over (eg for
-    ``itertools.chain``, which does not support the close attribute) an
-    instance of ``ClosingIterator`` is returned which will proxy calls to
-    ``close`` as necessary.
-    """
-
-    close = getattr(src, "close", marker)
-    if close is not marker:
-        try:
-            setattr(dst, "close", close)
-        except AttributeError:
-            return ClosingIterator(dst, close)
-
-    return dst
