@@ -14,7 +14,11 @@
 #
 from operator import attrgetter, itemgetter, methodcaller
 from fresco.exceptions import MissingRouteArg
+from fresco.request import Request
 from itertools import cycle
+from typing import Any
+from typing import Callable
+from typing import Mapping
 
 __all__ = (
     "routearg",
@@ -231,7 +235,7 @@ class RequestArg(RouteArg):
     session = attrgetter("session")
 
     #: Source for the request variable
-    source = form
+    source: Callable[[Request], Mapping] = form
 
     #: Exceptions that signal the converter could not do its job due to invalid
     #: input
@@ -246,6 +250,7 @@ class RequestArg(RouteArg):
     ):
         super(RequestArg, self).__init__(default, exception)
         self.formkey = key
+        self.getter: Callable[[Mapping], Any]
         self.is_list = isinstance(converter, list)
         if self.is_list:
             self.converter = lambda vs: [c(v) for c, v in zip(cycle(converter), vs)]
