@@ -25,14 +25,13 @@ from typing import Any
 from typing import Callable
 from typing import Dict
 from typing import List
-from typing import Iterator
-from typing import Iterable
 from typing import Mapping
 from typing import Optional
 from typing import Union
 from typing import Set
 from typing import Tuple
 from weakref import WeakKeyDictionary
+import typing as t
 
 from fresco.exceptions import ResponseException
 from fresco.response import Response
@@ -242,6 +241,8 @@ class Pattern(object):
     URL path.
     """
 
+    segments: t.List["PatternSegment"]
+
     def match(self, path):
         """
         Should return a tuple of ``(positional_arguments, keyword_arguments)``
@@ -683,7 +684,7 @@ class Route(object):
 
     def __init__(
         self,
-        pattern,
+        pattern: t.Union[str, Pattern],
         methods=None,
         view=None,
         kwargs=None,
@@ -696,7 +697,6 @@ class Route(object):
         provide_request: Optional[bool] = None,
         **_kwargs,
     ):
-
         """
         :param pattern:     A string that can be compiled into a path pattern
         :param methods:     The list of HTTP methods the view is bound to
@@ -772,7 +772,7 @@ class Route(object):
                 methods = [methods]
             else:
                 # Catch the common error of not providing a valid method
-                if not isinstance(methods, Iterable):
+                if not isinstance(methods, t.Iterable):
                     raise TypeError(
                         "HTTP methods must be specified as a string or iterable"
                     )
@@ -1187,7 +1187,7 @@ class RouteCollection(MutableSequence):
                         if r.instance is None:
                             r = r.bindto(item)
                         self.add_route(r)
-                elif isinstance(item, Iterable):
+                elif isinstance(item, t.Iterable):
                     for r in item:
                         self.add_route(r)
                 else:
@@ -1207,7 +1207,7 @@ class RouteCollection(MutableSequence):
     def __radd__(self, other):
         if isinstance(other, Route):
             return RouteCollection([other]) + self
-        elif isinstance(other, Iterable):
+        elif isinstance(other, t.Iterable):
             return RouteCollection(other) + self
         raise TypeError("Cannot add %r to %r" % (other, self))
 
@@ -1245,7 +1245,6 @@ class RouteCollection(MutableSequence):
             self.reinit_route_cache()
 
     def add_route(self, route):
-
         self.__routes__.append(route)
         if self._route_cache is not None:
             self.reinit_route_cache()
@@ -1366,7 +1365,7 @@ class RouteCollection(MutableSequence):
 
     def get_route_traversals(
         self, path: str, method: Optional[str], request: Optional[Request] = None
-    ) -> Iterator[RouteTraversal]:
+    ) -> t.Iterator[RouteTraversal]:
         """
         Generate RouteTraversals for routes matching the given path and
         method::
@@ -1482,7 +1481,7 @@ class RouteCollection(MutableSequence):
 
     def get_routes(
         self, path: str, method: str, request: Optional[Request] = None
-    ) -> Iterable[RouteTraversal]:
+    ) -> t.Iterable[RouteTraversal]:
         warnings.warn(
             "RouteCollection.get_routes is deprecated and will be removed "
             "in a future version. Use get_route_traversals instead",
@@ -1490,7 +1489,15 @@ class RouteCollection(MutableSequence):
         )
         return self.get_route_traversals(path, method, request)
 
-    def route(self, pattern, methods=None, view=None, *args, **kwargs):
+    def route(
+        self,
+        pattern: t.Union[str, Pattern],
+        methods: t.Optional[t.Union[str, t.Iterable[str]]] = None,
+        view: t.Optional[t.Union[str, t.Callable]] = None,
+        *args,
+        route_class: t.Optional[t.Type[Route]] = None,
+        **kwargs,
+    ):
         """
         Match a URL pattern to a view function. Can be used as a function
         decorator, in which case the ``view`` parameter should not be passed.
@@ -1503,7 +1510,7 @@ class RouteCollection(MutableSequence):
         Other parameters are as for the :class:`Route` constructor.
         """
         # Catch the common error of not providing a valid method
-        if methods and not isinstance(methods, Iterable):
+        if methods and not isinstance(methods, t.Iterable):
             raise TypeError("HTTP methods must be specified as a string or iterable")
 
         # Called as a decorator?
@@ -1517,16 +1524,25 @@ class RouteCollection(MutableSequence):
 
             return route_decorator
 
-        route = self.route_class(pattern, methods, view, *args, **kwargs)
+        cls = route_class if route_class is not None else self.route_class
+        route = cls(pattern, methods, view, *args, **kwargs)
         self.add_route(route)
         return route
 
-    def route_wsgi(self, path, wsgiapp, rewrite_script_name=True, *args, **kwargs):
+    def route_wsgi(
+        self,
+        path: str,
+        wsgiapp: t.Union[t.Callable, str],
+        rewrite_script_name: bool = True,
+        *args,
+        **kwargs,
+    ):
         """
         Route requests to ``path`` to the given WSGI application
 
         :param path: the mount point for the app
-        :param wsgiapp: the WSGI callable
+        :param wsgiapp: the WSGI callable, or the path to a callable in the
+                        format ``myapp.module.do_wsgi``.
         :param rewrite_script_name: if ``True`` (the default),
                                     the mount point specified by ``path`` will
                                     be shifted off ``PATH_INFO`` and into the
@@ -1541,16 +1557,17 @@ class RouteCollection(MutableSequence):
         else:
             ws_path = None
 
+        resolved_wsgi_app = None
+
         def fresco_wsgi_view(
+            request,
             path=path,
             ws_path=ws_path,
-            context=context,
             rewrite_script_name=rewrite_script_name,
             fake_start_response=fake_start_response,
             make_response=Response.from_wsgi,
-            wsgiapp=wsgiapp,
         ):
-            request = context.request
+            nonlocal resolved_wsgi_app
             environ = request.environ
             ws_SCRIPT_NAME = environ["SCRIPT_NAME"]
             shift_script_name = rewrite_script_name and not (
@@ -1562,17 +1579,38 @@ class RouteCollection(MutableSequence):
                 environ = environ.copy()
                 environ["SCRIPT_NAME"] = ws_SCRIPT_NAME + ws_path
                 environ["PATH_INFO"] = environ["PATH_INFO"][len(ws_path) :]
-            return make_response(wsgiapp, environ, fake_start_response)
 
-        return self.route_all(path, ALL_METHODS, fresco_wsgi_view, *args, **kwargs)
+            if resolved_wsgi_app is None:
+                if isinstance(wsgiapp, str):
+                    if "." in wsgiapp:
+                        mod_name, attr_name = wsgiapp.rsplit(".", 1)
+                        mod = import_module(mod_name)
+                        resolved_wsgi_app = getattr(mod, attr_name)
+                else:
+                    resolved_wsgi_app = wsgiapp
 
-    def route_all(self, path, *args, **kwargs):
-        """\
+            return make_response(resolved_wsgi_app, environ, fake_start_response)
+
+        return self.route_all(
+            path, ALL_METHODS, fresco_wsgi_view, route_class=RRoute, *args, **kwargs
+        )
+
+    def route_all(
+        self, path, methods=None, view=None, *args, route_class=None, **kwargs
+    ):
+        """
         Expose a view for all URLs starting with ``path``.
 
         :param path: the path prefix at which the view will be routed
         """
-        return self.route(MatchAllURLsPattern(path), *args, **kwargs)
+        return self.route(
+            MatchAllURLsPattern(path),
+            methods,
+            view,
+            route_class=route_class,
+            *args,
+            **kwargs,
+        )
 
     def include(self, path, views, fallthrough_on=None):
         """
