@@ -59,3 +59,9 @@ def module_level_function():
     """
     A module level function
     """
+
+
+def wsgi_app(environ, start_response):
+    start_response("200 OK", [("Content-Type", "text/plain")])
+
+    return [b"ok"]
