@@ -55,7 +55,5 @@ class TestCookie(object):
     def test_dates_are_rfc_formatted(self):
         c = Cookie("a", "b", expires=datetime(2001, 1, 1, 12))
         assert "Expires=Mon, 01 Jan 2001 12:00:00 GMT;" in str(c)
-        c = Cookie(
-            "a", "b", expires=datetime(2001, 1, 1, 12, tzinfo=FakeTZInfo())
-        )
+        c = Cookie("a", "b", expires=datetime(2001, 1, 1, 12, tzinfo=FakeTZInfo()))
         assert "Expires=Mon, 01 Jan 2001 11:00:00 GMT;" in str(c)
