@@ -41,7 +41,6 @@ class CustomException(Exception):
 
 class TestFrescoApp(object):
     def test_route_operates_as_a_decorator(self):
-
         app = FrescoApp()
 
         @app.route("/", GET)
@@ -88,7 +87,6 @@ class TestFrescoApp(object):
             assert app.view().status_code == 200
 
     def test_HEAD_request_delegated_to_GET_view(self):
-
         app = FrescoApp()
 
         @app.route("/", GET)
@@ -155,7 +153,6 @@ class TestFrescoApp(object):
             assert list(app.view().content_iterator) == [b"bar"]
 
     def test_wsgi_app_handles_response_exceptions(self):
-
         from fresco.exceptions import NotFound
 
         def view():
@@ -169,26 +166,17 @@ class TestFrescoApp(object):
 
     def test_route_wsgi_app(self):
         def wsgiapp(environ, start_response):
-
-            start_response(
-                "200 OK", [("Content-Type", "application/x-pachyderm")]
-            )
+            start_response("200 OK", [("Content-Type", "application/x-pachyderm")])
             return [b"pretty pink elephants"]
 
         app = FrescoApp()
         app.route_wsgi("/", wsgiapp)
 
         with app.requestcontext("/"):
-            assert list(app.view().content_iterator) == [
-                b"pretty pink elephants"
-            ]
-            assert (
-                app.view().get_header("Content-Type")
-                == "application/x-pachyderm"
-            )
+            assert list(app.view().content_iterator) == [b"pretty pink elephants"]
+            assert app.view().get_header("Content-Type") == "application/x-pachyderm"
 
     def test_get_methods_matches_on_path(self):
-
         app = FrescoApp()
         app.route("/1", POST, lambda: None)
         app.route("/1", PUT, lambda: None)
@@ -202,7 +190,6 @@ class TestFrescoApp(object):
             assert app.get_methods(c.request, "/2") == set([GET, DELETE])
 
     def test_get_methods_matches_on_predicate(self):
-
         p1 = Mock(return_value=True)
         p2 = Mock(return_value=False)
 
@@ -217,9 +204,7 @@ class TestFrescoApp(object):
 
     def test_invalid_path_encoding_triggers_bad_request(self):
         app = FrescoApp()
-        with app.requestcontext(
-            PATH_INFO=fixtures.misquoted_wsgi_unicode_path
-        ):
+        with app.requestcontext(PATH_INFO=fixtures.misquoted_wsgi_unicode_path):
             assert app.view().status_code == 400
 
     def test_remove_middleware(self):
@@ -234,7 +219,6 @@ class TestFrescoApp(object):
         assert m2.call_count == 1
 
     def test_insert_middleware(self):
-
         calls = []
 
         def middleware(app, name):
@@ -258,7 +242,6 @@ class TestFrescoApp(object):
         assert calls == ["mercury", "venus", "earth"]
 
     def test_routing_falls_though_with_fallthrough_on(self):
-
         app = FrescoApp()
         app.route(
             "/",
@@ -310,6 +293,22 @@ class TestAppHooks(object):
             assert app.view() is new_response
             assert view.call_count == 0
 
+    def test_process_request_once_only_called_once(self):
+        app = FrescoApp()
+        view = Mock(return_value=Response())
+        app.route("/", GET, view)
+        new_response = Response()
+
+        @app.process_request_once
+        def process_request(request):
+            return new_response
+
+        with app.requestcontext("/"):
+            assert app.view() is new_response
+            assert view.call_count == 0
+            assert app.view() is not new_response
+            assert view.call_count == 1
+
     def test_process_view_continues_on_none(self):
         app = FrescoApp()
         view = Mock(return_value=Response())
@@ -486,7 +485,6 @@ class TestAppHooks(object):
             app.process_teardown(Mock())
 
     def test_exception_in_process_hooks_do_not_stop_processing(self):
-
         for hook, view in [
             ("process_request", Response),
             ("process_view", Response),
@@ -499,7 +497,7 @@ class TestAppHooks(object):
             process_teardown = Mock(return_value=None)
 
             app = FrescoApp()
-            app.route("/", GET, view)
+            app.route("/", GET, view)  # type: ignore
             getattr(app, hook)(faulty_hook)
             app.process_teardown(process_teardown)
 
@@ -510,7 +508,6 @@ class TestAppHooks(object):
             assert process_teardown.call_count == 1
 
     def test_it_raises_exception_in_route_resolution(self):
-
         from fresco.routing import register_converter, StrConverter
 
         @register_converter("bugs")
@@ -546,7 +543,6 @@ class TestProcessException(object):
                 app.view()
 
     def test_it_reraises_if_no_500_error_handler_installed(self):
-
         app = FrescoApp()
         app.process_http_error_response(lambda req, res: None, 404)
 
@@ -666,7 +662,6 @@ class TestProcessException(object):
 
 class TestIncludeApp(object):
     def test_it_routes_to_an_included_app(self):
-
         app = FrescoApp()
 
         @app.route("/", GET)
@@ -693,9 +688,7 @@ class TestIncludeApp(object):
         app2.include("/app1", app)
 
         with app2.requestcontext("/app1/"):
-            assert list(app2.view().content_iterator) == [
-                b"http://localhost/app1/"
-            ]
+            assert list(app2.view().content_iterator) == [b"http://localhost/app1/"]
 
 
 class TestTrailingSlashes(object):
@@ -724,14 +717,12 @@ class TestTrailingSlashes(object):
 
 class TestViewCollection(object):
     def test_appdef(self):
-
         app = FrescoApp()
         app.include("/", fixtures.CBV("bananas!"))
         with app.requestcontext("/"):
             assert list(app.view().content_iterator) == [b"bananas!"]
 
     def test_appdef_url_generation(self):
-
         foo = fixtures.CBV("foo!")
         bar = fixtures.CBV("bar!")
         baz = fixtures.CBV("baz!")
@@ -746,7 +737,6 @@ class TestViewCollection(object):
             assert urlfor(baz.index_html) == "http://localhost/baz/"
 
     def test_instance_available_in_context(self):
-
         s = []
 
         class MyCBV(fixtures.CBV):
@@ -792,7 +782,6 @@ class TestAppRequestContext(object):
         return middleware
 
     def test_creates_isolated_context(self):
-
         app = FrescoApp()
         with app.requestcontext():
             context.request = "foo"
@@ -804,7 +793,6 @@ class TestAppRequestContext(object):
             assert context.request == "foo"
 
     def test_parses_full_url(self):
-
         with FrescoApp().requestcontext("https://arthur@example.org:123/?x=y"):
             assert context.request.environ["HTTPS"] == "on"
             assert context.request.environ["REMOTE_USER"] == "arthur"
@@ -848,21 +836,18 @@ class TestAppRequestContext(object):
             assert context.request.environ["CONTENT_TYPE"] == "foo"
 
     def test_it_invokes_middleware(self):
-
         app = FrescoApp()
         app.add_middleware(self._middleware)
         with app.requestcontext() as c:
             assert "sausages" in c.request.environ
 
     def test_it_skips_middleware(self):
-
         app = FrescoApp()
         app.add_middleware(self._middleware)
         with app.requestcontext(middleware=False) as c:
             assert "sausages" not in c.request.environ
 
     def test_it_closes_middleware(self):
-
         close = Mock()
 
         def middleware(app):
@@ -900,7 +885,6 @@ class TestAppRequestContext(object):
             assert next(counter) == 1
 
     def test_it_calls_late_added_middleware(self):
-
         calls = []
 
         def middleware(app):
@@ -929,7 +913,6 @@ class TestAppRequestContext(object):
         assert calls == ["middleware initialized", "middleware called"]
 
     def test_it_passes_a_valid_start_response_to_middleware(self):
-
         calls = []
 
         def middleware(app):
@@ -962,10 +945,7 @@ class TestUrlfor(object):
         app.route("/", GET, view)
         with app.requestcontext():
             assert urlfor(view) == "http://localhost/"
-            assert (
-                urlfor("fresco.tests.fixtures.aliased_view")
-                == "http://localhost/"
-            )
+            assert urlfor("fresco.tests.fixtures.aliased_view") == "http://localhost/"
 
         delattr(fixtures, "aliased_view")
 
@@ -985,9 +965,7 @@ class TestUrlfor(object):
         app = FrescoApp()
         app.route("/foo", GET, view)
         with app.requestcontext():
-            assert (
-                urlfor(view, _script_name="/abc") == "http://localhost/abc/foo"
-            )
+            assert urlfor(view, _script_name="/abc") == "http://localhost/abc/foo"
 
     def test_urlfor_with_string(self):
         app = FrescoApp()
@@ -1018,7 +996,6 @@ class TestUrlfor(object):
             assert urlfor(myviewfunc) == "http://localhost/1"
 
     def test_urlfor_with_class_based_view_spec(self):
-
         app = FrescoApp()
         app.include("/foo", fixtures.CBV("x"))
         with app.requestcontext():
@@ -1029,9 +1006,7 @@ class TestUrlfor(object):
 
     def test_it_uses_values_from_path_defaults(self):
         app = FrescoApp()
-        app.route(
-            "/<test:int>", GET, lambda: None, name="test", test_default=1
-        )
+        app.route("/<test:int>", GET, lambda: None, name="test", test_default=1)
         with app.requestcontext():
             assert urlfor("test") == "http://localhost/1"
 
@@ -1059,6 +1034,5 @@ class TestUrlfor(object):
             assert urlfor(f, _query="a=1") == "http://localhost/?a=1"
             assert urlfor(f, _query={"a": "1"}) == "http://localhost/?a=1"
             assert (
-                urlfor(f, _query=[("a", "1"), ("a", 2)])
-                == "http://localhost/?a=1&a=2"
+                urlfor(f, _query=[("a", "1"), ("a", 2)]) == "http://localhost/?a=1&a=2"
             )
