@@ -18,9 +18,7 @@ from fresco.decorators import json_response
 class TestJSONResponse(object):
     def test_it_allows_custom_formatting(self):
         r = json_response({"l": [1, 2, 3]}, indent=1, separators=(", ", ": "))
-        assert list(r.content_iterator) == [
-            b'{\n "l": [\n  1, \n  2, \n  3\n ]\n}'
-        ]
+        assert list(r.content_iterator) == [b'{\n "l": [\n  1, \n  2, \n  3\n ]\n}']
 
     def test_it_acts_as_a_filter(self):
         r = json_response({"l": [1, 2, 3]})
