@@ -18,9 +18,7 @@ from fresco.multidict import MultiDict
 
 def is_consistent(multidict: MultiDict) -> bool:
     "Verify that the MultiDict has internal consistency"
-    dictallitems = sorted(
-        (k, v) for k, vs in multidict._dict.items() for v in vs
-    )
+    dictallitems = sorted((k, v) for k, vs in multidict._dict.items() for v in vs)
     orderallitems = sorted(multidict._order)
     return list(sorted(dictallitems)) == list(sorted(orderallitems))
 
