@@ -25,6 +25,7 @@ import pytest
 
 from fresco.options import Options
 from fresco.options import parse_key_value_pairs
+from fresco.options import dict_from_options
 
 
 class TestOptions(object):
@@ -66,7 +67,6 @@ class TestOptions(object):
         assert "__module__" in options
 
     def test_options_update_from_file(self):
-
         with NamedTemporaryFile() as tmpfile:
             tmpfile.write(b"a = 1\nb = 2\n")
             tmpfile.flush()
@@ -76,7 +76,6 @@ class TestOptions(object):
             assert options == {"a": 1, "b": 2}
 
     def test_options_update_from_file_has_dunder_file_global(self):
-
         with NamedTemporaryFile() as tmpfile:
             tmpfile.write(b"a = __file__")
             tmpfile.flush()
@@ -144,7 +143,15 @@ class TestLoadKeyValuePairs:
 
 
 class TestLoadOptions:
+
     def check_loadoptions(self, tmpdir, files, sources="*", tags=[], expected={}):
+        """
+        Write the files indicated in ``sources`` to the given temporary directory,
+
+        Create an Options object and populate it from the specified sources/tags.
+
+        Assert that the loaded options is equal to the value of ``expected``.
+        """
         t = pathlib.Path(tmpdir)
         for fname in files:
             with (t / fname).open("w", encoding="UTF-8") as f:
@@ -153,7 +160,7 @@ class TestLoadOptions:
         @contextlib.contextmanager
         def optionsdir():
             def loadopts(sources="*", tags=[], strict=False, **kw):
-                return Options().load(str(sources), tags, strict=False, **kw)
+                return Options().load(sources, tags, strict=False, **kw)
 
             saved = os.getcwd()
             os.chdir(t)
@@ -215,10 +222,10 @@ class TestLoadOptions:
             assert loadopts("*", ["local", "dev"]) == {"a": "dev"}
 
     def test_it_loads_from_os_environ(self, tmpdir):
-        with self.check_loadoptions(tmpdir, {"a.txt": "a = 1"}) as loadopts:
-            os.environ["a"] = "2"
-            assert loadopts("*", [], use_environ=False) == {"a": 1}
-            assert loadopts("*", [], use_environ=True) == {"a": 2}
+        with setenv(a="2"):
+            with self.check_loadoptions(tmpdir, {"a.txt": "a = 1"}) as loadopts:
+                assert loadopts("*", [], use_environ=False) == {"a": 1}
+                assert loadopts("*", [], use_environ=True) == {"a": 2}
 
     def test_it_calls_callbacks(self, tmpdir):
         with self.check_loadoptions(tmpdir, {"a.txt": "a = 1"}):
@@ -234,3 +241,74 @@ class TestLoadOptions:
                 os.chdir(tmpdir2)
                 assert loadopts("*") == {}
                 assert loadopts("*", dir=tmpdir) == {"a": 1}
+
+    def test_it_accepts_a_list_of_filespecs(self, tmpdir):
+        self.check_loadoptions(
+            tmpdir,
+            {"a.txt": "a=1", "b.txt": "b=1"},
+            sources=["a.*", "b.*"],
+            expected={"a": 1, "b": 1}
+        )
+
+    def test_it_substitutes_from_environment_variables(self, tmpdir):
+        with setenv(FOO="bar"):
+            self.check_loadoptions(
+                tmpdir,
+                {"a.txt": "a=1", "a.bar.txt": "a=2"},
+                tags=["{FOO}"],
+                expected={"a": 2}
+            )
+
+        with setenv(FOO="baz"):
+            self.check_loadoptions(
+                tmpdir,
+                {"a.txt": "a=1", "a.bar.txt": "a=2"},
+                tags=["{FOO}"],
+                expected={"a": 1}
+            )
+
+    def test_it_allows_missing_environment_variables(self, tmpdir):
+        assert "FOO" not in os.environ
+        self.check_loadoptions(
+            tmpdir,
+            {"a.txt": "a=1", "a.bar.txt": "a=2"},
+            tags=["{FOO}"],
+            expected={"a": 1}
+        )
+
+
+class TestDictFromOptions:
+
+    def test_it_splits_on_prefix(self):
+
+        options = Options(FOO_BAR=1, FOO_BAZ=2, FOO_BAR_BAZ=3, BAR=4)
+        assert dict_from_options("FOO_", options) == {"BAR": 1, "BAZ": 2, "BAR_BAZ": 3}
+
+    def test_it_splits_recursively(self):
+
+        options = Options(
+            A_A=1,
+            A_B_C_D=2,
+            A_B_E=3,
+            A_F_G_H=4,
+            A_I=5,
+            J_A=6,
+        )
+        assert dict_from_options("A_", options, recursive=True) == {
+            "A": 1,
+            "B": {"C": {"D": 2}, "E": 3},
+            "F": {"G": {"H": 4}},
+            "I": 5
+        }
+
+
+@contextlib.contextmanager
+def setenv(**kw):
+    saved = {k: os.environ[k] for k in kw if k in os.environ}
+    os.environ.update(kw)
+    yield os.environ
+    for k in kw:
+        if k in saved:
+            os.environ[k] = saved[k]
+        else:
+            del os.environ[k]
