@@ -173,12 +173,10 @@ class TestRequestProperties(object):
 
 class TestPathEncoding(object):
     def test_url_is_quoted(self):
-
         with context(SCRIPT_NAME=fixtures.wsgi_unicode_path, PATH_INFO="") as c:
             assert c.request.url == "http://localhost" + fixtures.quoted_unicode_path
 
     def test_application_url_is_quoted(self):
-
         with context(SCRIPT_NAME=fixtures.wsgi_unicode_path, PATH_INFO="") as c:
             assert (
                 c.request.application_url
@@ -257,6 +255,17 @@ class TestMakeURL(object):
                 "http://localhost/?a=1&b=2+3",
             )
 
+    def test_query_drops_items_with_None_value(self):
+        with context() as c:
+            self.assert_equal_with_query(
+                c.request.make_url(query={"a": 1, "b": None}),
+                "http://localhost/?a=1",
+            )
+            self.assert_equal_with_query(
+                c.request.make_url(query=[("a", 1), ("b", None)]),
+                "http://localhost/?a=1",
+            )
+
     def test_query_kwargs(self):
         with context() as c:
             self.assert_equal_with_query(
@@ -364,7 +373,6 @@ class TestCurrentRequest(object):
 
 
 class TestMultipart(object):
-
     filename = "test.txt"
     filedata = "123456\n"
     boundary = "---------------------------1234"
@@ -392,29 +400,28 @@ class TestMultipart(object):
     }
 
     def test_files_populated(self):
-
         with FrescoApp().requestcontext(**self.request_args) as c:
             request = c.request
 
-        assert len(request.files) == 1
-        assert "uploaded_file" in request.files
+            assert len(request.files) == 1
+            assert "uploaded_file" in request.files
 
     def test_file_content_available(self):
-
         with FrescoApp().requestcontext(**self.request_args) as c:
             request = c.request
 
-        b = BytesIO()
-        request.files["uploaded_file"].save(b)
+            b = BytesIO()
+            request.files["uploaded_file"].save(b)
         assert b.getvalue() == self.filedata.encode("latin1")
 
     def test_headers_available(self):
-
         with FrescoApp().requestcontext(**self.request_args) as c:
             request = c.request
 
-        assert request.files["uploaded_file"].headers["content-type"] == "text/plain"
-        assert request.files["uploaded_file"].filename == self.filename
+            assert (
+                request.files["uploaded_file"].headers["content-type"] == "text/plain"
+            )
+            assert request.files["uploaded_file"].filename == self.filename
 
     def test_quotes_in_input_names_are_decoded(self):
         """
