@@ -185,7 +185,6 @@ class TestResponse(object):
         )
 
     def test_response_get_headers(self):
-
         r = Response(
             ["whoa nelly!"],
             content_type="text/plain",
@@ -200,14 +199,14 @@ class TestResponse(object):
         r = Response(["whoa nelly!"]).buffered()
         assert r.get_header("content-length") == "11"
 
-        r = Response([u"whoa nélly!"]).buffered()
+        r = Response(["whoa nélly!"]).buffered()
         assert r.get_header("content-length") == "12"
 
     def test_it_encodes_a_unicode_string_according_to_content_type(self):
-        r = Response(u"café", content_type="text/plain; charset=utf-8")
-        assert list(r({}, Mock())) == [u"café".encode("utf-8")]
-        r = Response(u"café", content_type="text/plain; charset=latin-1")
-        assert list(r({}, Mock())) == [u"café".encode("latin-1")]
+        r = Response("café", content_type="text/plain; charset=utf-8")
+        assert list(r({}, Mock())) == ["café".encode("utf-8")]
+        r = Response("café", content_type="text/plain; charset=latin-1")
+        assert list(r({}, Mock())) == ["café".encode("latin-1")]
 
 
 class TestAddVary(object):
