@@ -50,7 +50,6 @@ def assert_method_bound_to(method, ob):
 
 class TestMethodDispatch(object):
     def test_route_is_dispatched_to_correct_method(self):
-
         getview = Mock(return_value=Response())
         postview = Mock(return_value=Response())
         app = FrescoApp()
@@ -70,7 +69,6 @@ class TestMethodDispatch(object):
 
 class TestRouteConstructor(object):
     def test_multiple_views_can_be_associated_with_a_route(self):
-
         app = FrescoApp()
         v1 = Mock(return_value=Response())
         v2 = Mock(return_value=Response())
@@ -103,7 +101,6 @@ class TestRouteConstructor(object):
             assert v2.call_count == 1
 
     def test_it_catches_invalid_methods(self):
-
         # Route must be a string or an iterable
         with pytest.raises(TypeError):
             Route("/", object(), lambda: None)
@@ -162,7 +159,6 @@ class TestRouteBeforeHooks(object):
         assert mock.call_args_list == [call(foo="bar")]
 
     def test_it_applies_chained_hook_calls_in_order(self):
-
         mock = Mock(return_value=None)
 
         @Route.before(mock, 1)
@@ -240,7 +236,6 @@ class TestRouteDecorators(object):
         return exclaim
 
     def test_decorator_is_applied(self):
-
         views = fixtures.CBV("test")
 
         app = FrescoApp()
@@ -254,7 +249,6 @@ class TestRouteDecorators(object):
             assert list(app.view().content_iterator) == [b"test"]
 
     def test_decorator_is_applied_with_wrap_method(self):
-
         views = fixtures.CBV("test")
 
         app = FrescoApp()
@@ -288,7 +282,6 @@ class TestRouteDecorators(object):
             assert list(app.view().content_iterator) == [b"test!"]
 
     def test_decorator_works_with_urlfor(self):
-
         views = fixtures.CBV("test")
         app = FrescoApp()
         app.route("/decorated", GET, views.index_html, decorators=[self.exclaim])
@@ -373,7 +366,6 @@ class TestRouteCollection(object):
         assert [r.name for r in rc] == ["1", "2", "3"]
 
     def test_get_routes_matches_on_method(self):
-
         r_get = Route("/", GET, None)
         r_post = Route("/", POST, None)
 
@@ -383,7 +375,6 @@ class TestRouteCollection(object):
         assert [r.route for r in rc.get_route_traversals("/", POST)] == [r_post]
 
     def test_get_routes_matches_on_path(self):
-
         r1 = Route("/1", GET, None)
         r2 = Route("/2", GET, None)
 
@@ -393,7 +384,6 @@ class TestRouteCollection(object):
         assert [r.route for r in rc.get_route_traversals("/2", GET)] == [r2]
 
     def test_get_routes_can_match_all_methods(self):
-
         r1 = Route("/1", GET, None)
         r2 = Route("/1", POST, None)
 
@@ -404,7 +394,6 @@ class TestRouteCollection(object):
         ]
 
     def test_route_returns_traversal_information_on_nested_routes(self):
-
         a = RouteCollection()
         b = RouteCollection()
 
@@ -612,6 +601,13 @@ class TestRouteCollection(object):
         test_wsgi_app("/", "/foo", "", "/foo")
         test_wsgi_app("/prefix", "/prefix", "", "/prefix", rewrite_script_name=False)
 
+    def test_it_routes_to_a_wsgi_app_by_dotted_path(self):
+        app = FrescoApp()
+        app.route_wsgi("/", "fresco.tests.fixtures.wsgi_app")
+        with app.requestcontext("/"):
+            response = app.view()
+        assert list(response.content_iterator) == [b"ok"]
+
     def test_fallthrough_can_be_applied_to_collection(self):
         a = Route("/fee", GET=lambda: Response(status=204))
         a = Route("/fie", GET=lambda: Response(status=204))
@@ -677,7 +673,6 @@ class TestDelegatedRoutes(object):
             assert app.view().content == b"OK"
 
     def test_url_variables_are_passed(self):
-
         hello = Mock(return_value=Response())
 
         inner = FrescoApp()
@@ -691,7 +686,6 @@ class TestDelegatedRoutes(object):
             assert hello.call_args_list == [call(i="bar", o="foo")]
 
     def test_delegation_to_dynamic_routes(self):
-
         result = []
 
         class MyRoutes(object):
@@ -713,7 +707,6 @@ class TestDelegatedRoutes(object):
             assert inner_kwargs == {"inner": 2}
 
     def test_dynamic_routes_are_never_shared(self):
-
         result = []
 
         class MyRoutes(object):
@@ -748,7 +741,6 @@ class TestDelegatedRoutes(object):
             assert outer.pathfor("delegation:inner-route", o="x", i="y") == "/x/y"
 
     def test_pathfor_with_dynamic_delegated_route(self):
-
         view = Mock(return_value=Response())
 
         def routecollectionfactory(*args, **kwargs):
@@ -760,7 +752,6 @@ class TestDelegatedRoutes(object):
         assert rc.pathfor("delegation:inner-route", o="x", i="y") == "/x/y"
 
     def test_pathfor_with_dynamic_delegated_route_uses_default_args(self):
-
         view = Mock(return_value=Response())
 
         def routecollectionfactory(factoryarg1, factoryarg2):
@@ -779,7 +770,6 @@ class TestDelegatedRoutes(object):
         assert rc.pathfor("delegation:inner-route", i="y") == "/foo/bar/y"
 
     def test_urlfor_with_dynamic_delegated_route_and_view_self(self):
-
         result = []
 
         class MyRoutes(object):
@@ -814,7 +804,6 @@ class TestDelegatedRoutes(object):
             assert urlfor("x:y", a="a", b=1) == "http://localhost/a/1"
 
     def test_delegated_routes_can_be_included(self):
-
         view = Mock(return_value=Response())
 
         inner = RouteCollection([Route("/baz", GET, view)])
@@ -989,7 +978,6 @@ class TestRouteClassIsPluggable(object):
         ]
 
     def test_it_uses_route_class_in_decorator(self):
-
         routes = RouteCollection(route_class=self.CustomRoute)
 
         @routes.route("/", GET)
@@ -1016,7 +1004,6 @@ class TestRouteClassIsPluggable(object):
 
 
 class TestRouteTraversal(object):
-
     routes = RouteCollection()
     inner = RouteCollection()
     inner2 = RouteCollection()
@@ -1033,7 +1020,6 @@ class TestRouteTraversal(object):
         assert traversal.build_path() == "/1/2/3"
 
     def test_it_modifies_the_path(self):
-
         traversal = next(self.routes.get_route_traversals("/1/2/3", GET))
 
         # Modify the traversal and check we get a modified path back
