@@ -28,6 +28,7 @@ from fresco import Response
 from fresco.exceptions import RequestParseError
 from fresco.routing import POST
 from fresco.util.http import FileUpload
+from fresco.util.http import PostParser
 from fresco.util.http import encode_multipart
 from fresco.util.http import parse_parameters
 from fresco.util.http import parse_post
@@ -81,31 +82,29 @@ class TestParseMultipart(object):
                 "CONTENT_LENGTH": data["content_length"],
                 "CONTENT_TYPE": data["content_type"],
             }
-            parsed = sorted(list(parse_post(environ, io, "UTF-8")))
+            with PostParser(environ, io, "UTF-8") as parsed:
+                parsed = sorted(list(parsed))
 
-            assert [name for name, value in parsed] == [
-                "empty-text-input",
-                "file-upload",
-                "text-input-ascii",
-                "text-input-unicode",
-            ]
+                assert [name for name, value in parsed] == [
+                    "empty-text-input",
+                    "file-upload",
+                    "text-input-ascii",
+                    "text-input-unicode",
+                ]
 
-            assert parsed[0] == ("empty-text-input", "")
-            assert parsed[2] == ("text-input-ascii", "abcdef")
-            assert parsed[3] == (
-                "text-input-unicode",
-                b"\xce\xb1\xce\xb2\xce\xb3\xce\xb4".decode("utf8"),
-            )
+                assert parsed[0] == ("empty-text-input", "")
+                assert parsed[2] == ("text-input-ascii", "abcdef")
+                assert parsed[3] == (
+                    "text-input-unicode",
+                    b"\xce\xb1\xce\xb2\xce\xb3\xce\xb4".decode("utf8"),
+                )
 
-            fieldname, fileupload = parsed[1]
-            assert isinstance(fileupload, FileUpload)
-            assert fieldname == "file-upload"
-            assert fileupload.filename == "test.data"
-            assert (
-                fileupload.headers["content-type"]
-                == "application/octet-stream"
-            )
-            assert fileupload.file.read() == form_data.FILE_UPLOAD_DATA
+                fieldname, fileupload = parsed[1]
+                assert isinstance(fileupload, FileUpload)
+                assert fieldname == "file-upload"
+                assert fileupload.filename == "test.data"
+                assert fileupload.headers["content-type"] == "application/octet-stream"
+                assert fileupload.file.read() == form_data.FILE_UPLOAD_DATA
 
     def test_non_ascii_headers(self):
         """
@@ -127,10 +126,10 @@ class TestParseMultipart(object):
             "CONTENT_TYPE": "multipart/form-data; boundary=--XXXXXX",
         }
         f = BytesIO(data)
-        parsed = list(parse_post(environ, f, "UTF-8"))
-        fieldname, fileupload = parsed[0]
-        assert isinstance(fileupload, FileUpload)
-        assert fileupload.filename == "caf\uFFFD\uFFFD.txt"
+        with PostParser(environ, f, "UTF-8") as parsed:
+            fieldname, fileupload = list(parsed)[0]
+            assert isinstance(fileupload, FileUpload)
+            assert fileupload.filename == "caf\uFFFD\uFFFD.txt"
 
     def test_malformed_headers_raise_exceptions(self):
         # Missing '=' in parameter
@@ -198,14 +197,11 @@ class TestParseMultipart(object):
         app = FrescoApp()
         app.route("/", POST, view)
 
-        with app.requestcontext_post(
-            "/", multipart=True, data=[("f1", "x" * 200)]
-        ):
+        with app.requestcontext_post("/", multipart=True, data=[("f1", "x" * 200)]):
             assert app.view().status == "413 Payload Too Large"
 
 
 class TestParseFormEncodedData(object):
-
     char_latin1 = b"\xa3"
     char_utf8 = b"\xc2\xa3"
     char = char_latin1.decode("latin1")
@@ -255,7 +251,6 @@ class TestParseFormEncodedData(object):
             assert response.status == "400 Bad Request"
 
     def test_non_utf8_data_posted(self):
-
         data = b"char=" + quote(self.char_latin1).encode("ascii")
         env = {
             "REQUEST_METHOD": "POST",
@@ -269,7 +264,6 @@ class TestParseFormEncodedData(object):
             assert request.form["char"] == self.char
 
     def test_non_utf8_data_getted(self):
-
         data = "char=" + quote(self.char_latin1)
         env = {"REQUEST_METHOD": "GET", "QUERY_STRING": data}
 
@@ -283,15 +277,10 @@ class TestEncodeMultipart(object):
     def test_it_encodes_a_data_dict(self):
         data, headers = encode_multipart([("foo", "bar baf")])
         data = data.getvalue()
-        assert (
-            b'Content-Disposition: form-data; name="foo"\r\n\r\nbar baf'
-            in data
-        )
+        assert b'Content-Disposition: form-data; name="foo"\r\n\r\nbar baf' in data
 
     def test_it_encodes_a_file_tuple(self):
-        data, headers = encode_multipart(
-            files=[("foo", "foo.txt", "ascii", "bar")]
-        )
+        data, headers = encode_multipart(files=[("foo", "foo.txt", "ascii", "bar")])
         data = data.getvalue()
         expected = (
             b"Content-Disposition: form-data; "
@@ -304,7 +293,6 @@ class TestEncodeMultipart(object):
 
 
 class TestParseParameters(object):
-
     token = st.text(
         alphabet="".join(
             chr(c) for c in range(33, 127) if chr(c) not in '()<>@,;:\\/[]?="'
