@@ -37,9 +37,7 @@ def fq_path(ob):
     method
     """
     name = (
-        getattr(ob, "__qualname__", None)
-        or getattr(ob, "__name__", None)
-        or repr(ob)
+        getattr(ob, "__qualname__", None) or getattr(ob, "__name__", None) or repr(ob)
     )
     module = getattr(ob, "__module__", None) or ""
     return "{}.{}".format(module, name)
