@@ -32,6 +32,8 @@ from typing import List
 from typing import Tuple
 from typing import Callable
 from typing import Union
+from typing import Optional
+import typing as t
 from tempfile import SpooledTemporaryFile
 import os
 import re
@@ -55,6 +57,8 @@ CHUNK_SIZE = min(io.DEFAULT_BUFFER_SIZE, 1024)
 
 ParsedContentType = namedtuple("ParsedContentType", "content_type encoding params")
 
+ParsedField = Union["FileUpload", str]
+
 token_pattern = r"[!#-\'*-.0-9A-Z\^-~]+"
 quotedstringparts_pattern = r'(?:(\\.)|([^"\\]+))'
 quotedstring_pattern = r'"(?:{})*"'.format(quotedstringparts_pattern)
@@ -195,9 +199,10 @@ def parse_header(
 
 def parse_querystring(
     data: str,
-    charset: str = None,
+    charset: Optional[str] = None,
     strict: bool = False,
     keep_blank_values: bool = True,
+    unquote_plus=unquote_plus,
 ) -> List[Tuple[str, str]]:
     """
     Return ``(key, value)`` pairs from the given querystring::
@@ -245,12 +250,12 @@ def parse_querystring(
 
 def parse_post(
     environ,
-    fp,
-    default_charset=None,
+    _io: t.IO[bytes],
+    default_charset: Optional[str] = None,
     max_size=16 * KB,
     max_multipart_size=2 * MB,
     ie_workaround=True,
-) -> Iterable[Tuple[str, Union["FileUpload", str]]]:
+) -> Tuple[Iterable[Tuple[str, ParsedField]], Optional[Callable]]:
     """\
     Parse the contents of an HTTP POST request, which may be either
     application/x-www-form-urlencoded or multipart/form-data encoded.
@@ -279,11 +284,14 @@ def parse_post(
         raise MissingContentLength()
 
     try:
-        bytestream = io_iterator(fp, CHUNK_SIZE, maxlen=content_length)
+        bytestream = io_iterator(_io, CHUNK_SIZE, maxlen=content_length)
         if ct == "application/x-www-form-urlencoded":
             if content_length > max_size:
                 raise TooBig("Content Length exceeds permitted size")
-            return parse_querystring(b"".join(bytestream).decode("ASCII"), charset)
+            return (
+                parse_querystring(b"".join(bytestream).decode("ASCII"), charset),
+                None,
+            )
         else:
             if content_length > max_multipart_size:
                 raise TooBig("Content Length exceeds permitted size")
@@ -304,6 +312,22 @@ def parse_post(
         raise RequestParseError("Payload contains non ascii data")
 
 
+class PostParser:
+    close: t.Optional[Callable] = None
+
+    def __init__(self, *args, **kwargs):
+        self.args = args
+        self.kwargs = kwargs
+
+    def __enter__(self):
+        items, self.close = parse_post(*self.args, **self.kwargs)
+        return items
+
+    def __exit__(self, exc_type, exc_value, exc_traceback):
+        if self.close:
+            self.close()
+
+
 def get_body_bytes(environ, max_size=16 * KB) -> bytes:
     """
     Read a single message body from environ['wsgi.input'], returning a bytes
@@ -319,16 +343,6 @@ def get_body_bytes(environ, max_size=16 * KB) -> bytes:
     return b"".join(io_iterator(environ["wsgi.input"], maxlen=content_length))
 
 
-def parse_body(environ, fp, default_charset=None, max_size=16 * KB):
-    """
-    Parse the message
-        (payload as a byte string, content-type, encoding)
-    """
-    ct, charset, params = get_content_type_info(
-        environ.get("CONTENT_TYPE", "text/plain")
-    )
-
-
 class HTTPMessage(Message):
     """
     Represent HTTP request message headers
@@ -337,15 +351,18 @@ class HTTPMessage(Message):
 
 def parse_multipart(
     stream: ByteIterator, boundary, default_charset, max_size, ie_workaround=True
-) -> Iterator[Tuple[str, Union["FileUpload", str]]]:
+) -> Tuple[Iterable[Tuple[str, ParsedField]], Optional[Callable]]:
     """
-    Parse data encoded as ``multipart/form-data``. Generate tuples of::
+    Parse data encoded as ``multipart/form-data``.
+    Return an iterator over tuples of ` (<field-name>, <data>)``, and an
+    optional ``close`` function.
 
-        (<field-name>, <data>)
-
-    Where ``data`` will be a string in the case of a regular input field, or a
+    ``data`` will be a string in the case of a regular input field, or a
     ``FileUpload`` instance if a file was uploaded.
 
+    If a ``close`` function is returned, the caller must call it in order to
+    close any temporary files created at the end of the request lifecycle.
+
     :param stream: input stream from which to read data
     :param boundary: multipart boundary string, as specified by the
                      ``Content-Disposition`` header
@@ -382,48 +399,69 @@ def parse_multipart(
         raise RequestParseError("Malformed POST data: expected CRLF")
 
     stream = chain([peek[boundary_size + 4 :]], stream)
+    open_files = set()
+    fields = []
 
-    while True:
-        headers, data, stream = _read_multipart_field(stream, boundary, max_size)
-        try:
-            _, params = parse_header(
-                headers["Content-Disposition"], ie_workaround=ie_workaround
-            )
-        except KeyError:
-            raise RequestParseError("Missing Content-Disposition header")
-
-        try:
-            name = params["name"]
-        except KeyError:
-            raise RequestParseError("Missing name in Content-Disposition header")
-
-        is_file_upload = "Content-Type" in headers and "filename" in params
-        if is_file_upload:
-            data.seek(0)
-            yield name, FileUpload(params["filename"], headers, data)
-        else:
-            charset = parse_header(headers.get("Content-Type", ""))[1].get(
-                "charset", default_charset
-            )
-            if data.tell() > max_size:
-                data.close()
-                raise TooBig("Data block exceeds maximum permitted size")
+    try:
+        while True:
+            headers, data, stream = _read_multipart_field(stream, boundary, max_size)
+            open_files.add(data)
             try:
-                data.seek(0)
-                yield name, data.read().decode(charset)
-            except UnicodeDecodeError:
-                raise RequestParseError(f"Invalid {charset} character data")
+                _, params = parse_header(
+                    headers["Content-Disposition"], ie_workaround=ie_workaround
+                )
+            except KeyError:
+                raise RequestParseError("Missing Content-Disposition header")
 
-        peek = next(stream)
-        if peek[:2] == b"\r\n":
-            stream = chain([peek[2:]], stream)
-        elif peek == b"--\r\n":
-            if next(stream, None) is None:
-                break
+            try:
+                name = params["name"]
+            except KeyError:
+                raise RequestParseError("Missing name in Content-Disposition header")
+
+            is_file_upload = "Content-Type" in headers and "filename" in params
+            if is_file_upload:
+                data.seek(0)
+                fu = FileUpload(params["filename"], headers, data)
+                fields.append((name, fu))
             else:
-                RequestParseError("Boundary not properly terminated")
-        else:
-            raise RequestParseError("Boundary not properly terminated")
+                charset = parse_header(headers.get("Content-Type", ""))[1].get(
+                    "charset", default_charset
+                )
+                if data.tell() > max_size:
+                    data.close()
+                    open_files.remove(data)
+                    raise TooBig("Data block exceeds maximum permitted size")
+                try:
+                    data.seek(0)
+                    fields.append((name, data.read().decode(charset)))
+                    data.close()
+                    open_files.remove(data)
+                except UnicodeDecodeError:
+                    raise RequestParseError(f"Invalid {charset} character data")
+
+            peek = next(stream)
+            if peek[:2] == b"\r\n":
+                stream = chain([peek[2:]], stream)
+            elif peek == b"--\r\n":
+                if next(stream, None) is None:
+                    break
+                else:
+                    RequestParseError("Boundary incorrectly terminated")
+            else:
+                raise RequestParseError("Boundary incorrectly terminated")
+    except Exception:
+        for f in open_files:
+            f.close()
+        raise
+
+    close: Optional[Callable] = None
+    if open_files:
+
+        def close():
+            for f in open_files:
+                f.close()
+
+    return fields, close
 
 
 def _read_multipart_field(
@@ -436,7 +474,7 @@ def _read_multipart_field(
     Iterator ``iostream`` must be positioned at the start of the header block
     for the field.
 
-    Return a tuple of ('<headers>', '<data>')
+    The caller must call ``data.close()`` after consuming the data.
 
     ``headers`` is an instance of ``email.message.Message``.
 
@@ -482,7 +520,7 @@ def read_until(
     exhausted.
     """
     buf = b""
-    found = None
+    found: t.Optional[bool] = None
 
     def _found():
         if found is None:
@@ -528,7 +566,6 @@ class FileUpload(object):
     """
 
     def __init__(self, filename, headers, fileob):
-
         self.filename = filename
         self.headers = headers
         self.file = fileob
