@@ -17,16 +17,15 @@
 Utilities for wrapping IO streams. These are used internally by
 fresco when parsing wsgi request input streams.
 """
-from io import IOBase
 from io import DEFAULT_BUFFER_SIZE
 
-from typing import Iterator
+import typing as t
 
 
-ByteIterator = Iterator[bytes]
+ByteIterator = t.Iterator[bytes]
 
 
-def io_iterator(fp: IOBase, size=DEFAULT_BUFFER_SIZE, maxlen=-1) -> ByteIterator:
+def io_iterator(fp: t.IO, size=DEFAULT_BUFFER_SIZE, maxlen=-1) -> ByteIterator:
     bytecount = 0
     if maxlen > 0:
         while True:
