@@ -43,7 +43,7 @@ def plural(quantity, one, multiple, zero=None, two=None):
         ...     multiple='%d green bottles, hanging on the wall',
         ... )
         >>> for bottles in range(10, -1, -1):
-        ...     print p(bottles)
+        ...     print(p(bottles))
         ...
         10 green bottles, hanging on the wall
         9 green bottles, hanging on the wall
