@@ -16,6 +16,7 @@
 Utilities for interfacing with WSGI
 """
 
+import logging
 from functools import partial
 from io import BytesIO
 from urllib.parse import unquote
@@ -29,6 +30,9 @@ from fresco.typing import ExcInfoTuple
 from fresco.typing import HeadersList
 from fresco.typing import WSGICallable
 
+logger = logging.getLogger(__name__)
+
+
 __all__ = [
     "environ_to_str",
     "str_to_environ",
@@ -301,9 +305,14 @@ class ClosingIterator(object):
         """
         Call all close functions listed in ``*close_funcs``.
         """
-        self._closed = True
         for func in self._close_funcs:
-            func()
+            try:
+                func()
+            except Exception:
+                logger.exception(
+                    "ContentIterator close function {func!r} raised an exception"
+                )
+        self._closed = True
 
     def __del__(self):
         """
